@@ -44,7 +44,7 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
     verbose : bool, optional
         Whether or not to print debugging statements
     sort_dims : bool
-        If True, the data is sorted so that the dimensions are in order from fastest to slowest
+        If True, the data is sorted so that the dimensions are in order from slowest to fastest
         If False, the data is kept in the original order
         If `get_labels` is also True, the labels are sorted as well.
     lazy : bool, optional. Default = False
@@ -81,18 +81,18 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
         if not check_if_main(h5_main):
             raise ValueError('if h5_main is a h5py.Dataset it should be a Main dataset')
     else:
-        if not isinstance(h5_main, (h5py.Dataset, np.ndarray)):
+        if not isinstance(h5_main, (h5py.Dataset, np.ndarray, da.core.Array)):
             raise TypeError('h5_main should either be a h5py.Dataset or numpy array')
 
     if h5_pos is not None:
-        if not isinstance(h5_pos, (h5py.Dataset, np.ndarray)):
+        if not isinstance(h5_pos, (h5py.Dataset, np.ndarray, da.core.Array)):
             raise TypeError('h5_pos should either be a h5py.Dataset or numpy array')
         if h5_pos.shape[0] != h5_main.shape[0]:
             raise ValueError('The size of h5_pos: {} does not match with h5_main: {}'.format(h5_pos.shape,
                                                                                              h5_main.shape))
 
     if h5_spec is not None:
-        if not isinstance(h5_spec, (h5py.Dataset, np.ndarray)):
+        if not isinstance(h5_spec, (h5py.Dataset, np.ndarray, da.core.Array)):
             raise TypeError('h5_spec should either be a h5py.Dataset or numpy array')
         if h5_spec.shape[1] != h5_main.shape[1]:
             raise ValueError('The size of h5_spec: {} does not match with h5_main: {}'.format(h5_spec.shape,
@@ -125,7 +125,7 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
         """
         ds_pos = h5_pos[()]
         pos_labs = get_attr(h5_pos, 'labels')
-    elif isinstance(h5_pos, np.ndarray):
+    elif isinstance(h5_pos, (np.ndarray, da.core.Array)):
         ds_pos = np.atleast_2d(h5_pos)
         pos_labs = np.array(['Position Dimension {}'.format(ipos) for ipos in range(ds_pos.shape[1])])
     else:
@@ -157,7 +157,7 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
         """
         ds_spec = h5_spec[()]
         spec_labs = get_attr(h5_spec, 'labels')
-    elif isinstance(h5_spec, np.ndarray):
+    elif isinstance(h5_spec, (np.ndarray, da.core.Array)):
         ds_spec = h5_spec
         spec_labs = np.array(['Spectral Dimension {}'.format(ispec) for ispec in range(ds_spec.shape[0])])
     else:
@@ -181,6 +181,19 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
     pos_dims = get_dimensionality(np.transpose(ds_pos), pos_sort)
     spec_dims = get_dimensionality(ds_spec, spec_sort)
 
+    if np.prod(pos_dims) != h5_main.shape[0]:
+        mesg = 'Product of position dimension sizes: {} = {} not matching ' \
+               'with size of first axis of main dataset: {}. One or more ' \
+               'dimensions are dependent dimensions and not marked as such' \
+               '.'.format(pos_dims, np.prod(pos_dims), h5_main.shape[0])
+        raise ValueError(mesg)
+    if np.prod(spec_dims) != h5_main.shape[1]:
+        mesg = 'Product of spectroscopic dimension sizes: {} = {} not matching ' \
+               'with size of second axis of main dataset: {}. One or more ' \
+               'dimensions are dependent dimensions and not marked as such' \
+               '.'.format(spec_dims, np.prod(spec_dims), h5_main.shape[1])
+        raise ValueError(mesg)
+
     if verbose:
         print('\nPosition dimensions (sort applied):', pos_labs[pos_sort])
         print('Position dimensionality (sort applied):', pos_dims)
@@ -198,6 +211,9 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
     Since the sort orders we have are from fastest to slowest, we need to reverse the orders
     for both the position and spectroscopic dimensions
     """
+    if verbose:
+        print('Will attempt to reshape main dataset from:\n{} to {}'.format(ds_main.shape, pos_dims[::-1] + spec_dims[::-1]))
+
     try:
         ds_Nd = ds_main.reshape(pos_dims[::-1] + spec_dims[::-1])
 
@@ -224,41 +240,35 @@ def reshape_to_n_dims(h5_main, h5_pos=None, h5_spec=None, get_labels=False, verb
                             spec_labs[spec_sort][::-1]))
 
     if verbose:
-        print('\nAfter first reshape, labels are', all_labels)
+        print('\nAfter reshaping, labels are', all_labels)
         print('Data shape is', ds_Nd.shape)
 
     """
-    Now we transpose the axes for both the position and spectroscopic dimensions
-    so that they are in the same order as in the index array
+    At this point, the data is arranged from slowest to fastest dimension in both pos and spec
     """
+    if sort_dims:
+        results = [ds_Nd, True]
+        if get_labels:
+            results.append(all_labels)
+        return results
+
+    if verbose:
+        print('\nGoing to put dimensions back in the same order as in the file:')
 
     swap_axes = list()
-    if sort_dims:
-        for lab in pos_labs[pos_sort]:
-            swap_axes.append(np.argwhere(all_labels == lab).squeeze())
-        for lab in spec_labs[spec_sort]:
-            swap_axes.append(np.argwhere(all_labels == lab).squeeze())
-    else:
-        for lab in pos_labs:
-            swap_axes.append(np.argwhere(all_labels == lab).squeeze())
-        for lab in spec_labs:
-            swap_axes.append(np.argwhere(all_labels == lab).squeeze())
+    # Compare the original order of the pos / spec labels with where these dimensions occur in the sorted labels
+    for lab in pos_labs:
+        swap_axes.append(np.argwhere(all_labels == lab).squeeze())
+    for lab in spec_labs:
+        swap_axes.append(np.argwhere(all_labels == lab).squeeze())
 
     swap_axes = np.array(swap_axes)
 
     if verbose:
-        print('\nAxes will permuted in this order:', swap_axes)
+        print('Axes will permuted in this order:', swap_axes)
         print('New labels ordering:', all_labels[swap_axes])
 
-    # Get a numeric version of the numpy version for comparisons
-    np_ver = np.__version__.split('.')[:2]
-    np_ver = float(np_ver[0] + '.' + np_ver[1])
-
-    if isinstance(ds_Nd, np.ndarray) and np_ver > 1.16:
-        ds_Nd = ds_Nd.transpose(tuple(swap_axes))[0]
-    else:
-        ds_Nd = ds_Nd.transpose(tuple(swap_axes))
-
+    ds_Nd = ds_Nd.transpose(tuple(swap_axes))
     results = [ds_Nd, True]
 
     if verbose:
@@ -471,6 +481,8 @@ def get_dimensionality(ds_index, index_sort=None):
         Dimensionality of each row in ds_index.  If index_sort is supplied, it will be in the sorted order
 
     """
+    if isinstance(ds_index, da.core.Array):
+        ds_index = ds_index.compute()
     if not isinstance(ds_index, (np.ndarray, h5py.Dataset)):
         raise TypeError('ds_index should either be a numpy array or h5py.Dataset')
 
@@ -513,6 +525,8 @@ def get_sort_order(ds_spec):
         Order of rows sorted from fastest changing to slowest
 
     """
+    if isinstance(ds_spec, da.core.Array):
+        ds_spec = ds_spec.compute()
     if not isinstance(ds_spec, (np.ndarray, h5py.Dataset)):
         raise TypeError('ds_spec should either be a numpy array or h5py.Dataset')
 
@@ -654,6 +668,11 @@ def get_unit_values(ds_inds, ds_vals, dim_names=None, all_dim_names=None, is_spe
         inds_for_dim = inds_mat[desired_row_ind]
         # Wherever this dimension goes to 0 - start of a new tile
         starts = np.where(inds_for_dim == np.min(inds_for_dim))[0]
+        if starts[0] != 0:
+            raise ValueError('Spectroscopic Indices for dimension: "{}" not '
+                             'starting with 0. Please fix this and try again'
+                             '.'.format(dim_name))
+
         # There may be repetitions in addition to tiling. Find how the the positions increase.
         # 1 = repetition, > 1 = new tile
         step_sizes = np.hstack(([1], np.diff(starts)))
@@ -700,7 +719,8 @@ def get_unit_values(ds_inds, ds_vals, dim_names=None, all_dim_names=None, is_spe
 
 def write_main_dataset(h5_parent_group, main_data, main_data_name, quantity, units, pos_dims, spec_dims,
                        main_dset_attrs=None, h5_pos_inds=None, h5_pos_vals=None, h5_spec_inds=None, h5_spec_vals=None,
-                       aux_spec_prefix='Spectroscopic_', aux_pos_prefix='Position_', verbose=False, **kwargs):
+                       aux_spec_prefix='Spectroscopic_', aux_pos_prefix='Position_', verbose=False,
+                       slow_to_fast=False, **kwargs):
     """
     Writes the provided data as a 'Main' dataset with all appropriate linking.
     By default, the instructions for generating the ancillary datasets should be specified using the pos_dims and
@@ -745,6 +765,9 @@ def write_main_dataset(h5_parent_group, main_data, main_data_name, quantity, uni
         Default prefix for Position datasets. Default = "Position"
     verbose : bool, Optional, default=False
         If set to true - prints debugging logs
+    slow_to_fast : bool, Optional. Default=False
+        Set to True if the dimensions are arranged from slowest varying to fastest varying.
+        Set to False otherwise.
     kwargs will be passed onto the creation of the dataset. Please pass chunking, compression, dtype, and other
         arguments this way
 
@@ -820,7 +843,7 @@ def write_main_dataset(h5_parent_group, main_data, main_data_name, quantity, uni
         if verbose:
             print('Passed all pre-tests for creating position datasets')
         h5_pos_inds, h5_pos_vals = write_ind_val_dsets(h5_parent_group, pos_dims, is_spectral=False, verbose=verbose,
-                                                       base_name=aux_pos_prefix)
+                                                       slow_to_fast=slow_to_fast, base_name=aux_pos_prefix)
         if verbose:
             print('Created position datasets!')
 
@@ -836,7 +859,7 @@ def write_main_dataset(h5_parent_group, main_data, main_data_name, quantity, uni
         if verbose:
             print('Passed all pre-tests for creating spectroscopic datasets')
         h5_spec_inds, h5_spec_vals = write_ind_val_dsets(h5_parent_group, spec_dims, is_spectral=True, verbose=verbose,
-                                                         base_name=aux_spec_prefix)
+                                                         slow_to_fast=slow_to_fast, base_name=aux_spec_prefix)
         if verbose:
             print('Created Spectroscopic datasets')
 
