@@ -176,6 +176,13 @@ def check_and_link_ancillary(h5_dset, anc_names, h5_main=None, anc_refs=None):
     """
     if not isinstance(h5_dset, h5py.Dataset):
         raise TypeError('h5_dset should be a h5py.Dataset object')
+
+    if isinstance(anc_names, (str, unicode)):
+        anc_names = [anc_names]
+    if isinstance(anc_refs, (h5py.Dataset, h5py.Group, h5py.File,
+                             h5py.Reference)):
+        anc_refs = [anc_refs]
+
     if not isinstance(anc_names, (list, tuple)):
         raise TypeError('anc_names should be a list / tuple')
     if h5_main is not None:
@@ -185,10 +192,13 @@ def check_and_link_ancillary(h5_dset, anc_names, h5_main=None, anc_refs=None):
         if not isinstance(anc_refs, (list, tuple)):
             raise TypeError('anc_refs should be a list / tuple')
 
+    if anc_refs is None and h5_main is None:
+        raise ValueError('No objected provided to link as ancillary')
+
     def __check_and_link_single(h5_obj_ref, target_ref_name):
         if isinstance(h5_obj_ref, h5py.Reference):
             h5_dset.attrs[target_ref_name] = h5_obj_ref
-        elif isinstance(h5_obj_ref, h5py.Dataset):
+        elif isinstance(h5_obj_ref, (h5py.Dataset, h5py.Group, h5py.File)):
             h5_dset.attrs[target_ref_name] = h5_obj_ref.ref
         elif h5_main is not None:
             h5_anc = get_auxiliary_datasets(h5_main, aux_dset_name=[target_ref_name])
@@ -921,7 +931,8 @@ def check_for_matching_attrs(h5_obj, new_parms=None, verbose=False):
     return all(tests)
 
 
-def write_ind_val_dsets(h5_parent_group, dimensions, is_spectral=True, verbose=False, base_name=None):
+def write_ind_val_dsets(h5_parent_group, dimensions, is_spectral=True, verbose=False, base_name=None,
+                        slow_to_fast=False):
     """
     Creates h5py.Datasets for the position OR spectroscopic indices and values of the data.
     Remember that the contents of the dataset can be changed if need be after the creation of the datasets.
@@ -941,6 +952,9 @@ def write_ind_val_dsets(h5_parent_group, dimensions, is_spectral=True, verbose=F
         Whether or not to print statements for debugging purposes
     base_name : str or unicode, optional
         Prefix for the datasets. Default: 'Position' when is_spectral is False, 'Spectroscopic' otherwise
+    slow_to_fast : bool, Optional. Default=False
+        Set to True if the dimensions are arranged from slowest varying to fastest varying.
+        Set to False otherwise.
 
     Returns
     -------
@@ -978,6 +992,9 @@ def write_ind_val_dsets(h5_parent_group, dimensions, is_spectral=True, verbose=F
         if is_spectral:
             base_name = 'Spectroscopic_'
 
+    if not slow_to_fast:
+        warn('In the future write_ind_val_dsets will default to requiring dimensions to be arranged from slowest to fastest varying')
+
     # check if the datasets already exist. If they do, there's no point in going any further
     for sub_name in ['Indices', 'Values']:
         if base_name + sub_name in h5_parent_group.keys():
@@ -992,8 +1009,18 @@ def write_ind_val_dsets(h5_parent_group, dimensions, is_spectral=True, verbose=F
     sing_mode = sing_mode[0]
 
     if sing_mode == DimType.DEFAULT:
+        if slow_to_fast:
+            # Ensure that the dimensions are arranged from fast to slow instead
+            dimensions = dimensions[::-1]
         indices, values = build_ind_val_matrices([dim.values for dim in dimensions],
                                                  is_spectral=is_spectral)
+        # At this point, dimensions and unit values are arranged from fastest to slowest
+        # We want dimensions to be arranged from slowest to fastest:
+        rev_func = np.flipud if is_spectral else np.fliplr
+        dimensions = dimensions[::-1]
+        indices = rev_func(indices)
+        values = rev_func(values)
+
     elif sing_mode == DimType.INCOMPLETE:
         lengths = np.unique([len(dim.values) for dim in dimensions])
         if len(lengths) > 1:
