@@ -7,7 +7,8 @@ Created on 7/17/16 10:08 AM
 @author: Suhas Somnath, Chris Smith
 """
 
-from __future__ import division, print_function, absolute_import
+from __future__ import division, unicode_literals, print_function, \
+    absolute_import
 import numpy as np
 import psutil
 import time as tm
@@ -15,12 +16,16 @@ import h5py
 from numbers import Number
 from multiprocessing import cpu_count
 
-from .comp_utils import parallel_compute, get_MPI, group_ranks_by_socket, get_available_memory
+from .comp_utils import parallel_compute, get_MPI, group_ranks_by_socket, \
+    get_available_memory
 from ..io.hdf_utils import check_if_main, check_for_old, get_attributes
 from ..io.usi_data import USIDataset
-from ..io.dtype_utils import integers_to_slices
+from ..io.dtype_utils import integers_to_slices, lazy_load_array, \
+    validate_single_string_arg
 from ..io.io_utils import format_time, format_size
 
+# TODO: internalize as many attributes as possible. Expose only those that will be required by the user
+
 
 class Process(object):
     """
@@ -29,13 +34,16 @@ class Process(object):
     only need to specify application-relevant code for processing the data.
     """
 
-    def __init__(self, h5_main, cores=None, max_mem_mb=4*1024,
-                 mem_multiplier=1.0, verbose=False):
+    def __init__(self, h5_main, process_name, parms_dict=None, cores=None,
+                 max_mem_mb=4*1024, mem_multiplier=1.0, lazy=False,
+                 verbose=False):
         """
         Parameters
         ----------
         h5_main : :class:`~pyUSID.io.usi_data.USIDataset`
             The USID main HDF5 dataset over which the analysis will be performed.
+         self.process_name : str
+            Name of the process
         cores : uint, optional
             How many cores to use for the computation. Default: all available cores - 2 if operating outside MPI context
         max_mem_mb : uint, optional
@@ -50,8 +58,76 @@ class Process(object):
             the size of results datasets as well. For example, if the result
             dataset is the same size and precision as the source dataset,
             the multiplier will be 2 (1 for source, 1 for result)
+        lazy : bool, optional. Default = False
+            If True, read_data_chunk and write_results_chunk will operate on
+            dask arrays. If False - everything will be in numpy.
         verbose : bool, Optional, default = False
             Whether or not to print debugging statements
+
+        Attributes
+        ----------
+        self.h5_results_grp : :class:`h5py.Group`
+            HDF5 group containing the HDF5 datasets that contain the results
+            of the computation
+        self.verbose : bool
+            Whether or not to print debugging statements
+        self.parms_dict : dict
+            Dictionary of parameters for the computation
+        self.duplicate_h5_groups : list
+            List of :class:`h5py.Group` objects containing computational
+            results that have been completely computed with the same
+            set of parameters as those in self.parms_dict
+        self.partial_h5_groups : list
+            List of :class:`h5py.Group` objects containing computational
+            results that have been partially computed with the same
+            set of parameters as those in self.parms_dict
+        self.process_name : str
+            Name of the process. This is used for checking for existing
+            completely and partially computed results as well as for naming
+            the HDF5 group that will contain the results of the computation
+        self._cores : uint
+            Number of CPU cores to use for parallel computations.
+            Ignored in the MPI context. Each rank gets 1 CPU core
+        self._max_pos_per_read : uint
+            Number of positions in the dataset to read per chunk
+        self._status_dset_name : str
+            Name of the HDF5 dataset that keeps track of the positions in the
+            source dataset thave already been computed
+        self._results : list
+            List of objects returned as the result of computation performed by
+            the self._map_function for each position in the current batch of
+            positions that were processed
+        self.__resume_implemented : bool
+            Whether or not this (child) class has implemented the
+            self._get_existing_datasets() function
+        self.__bytes_per_pos : uint
+            Number of bytes used by one position of the source dataset
+        self.mpi_comm : :class:`mpi4py.MPI.COMM_WORLD`
+            MPI communicator. None if not running in an MPI context
+        self.mpi_rank: uint
+            MPI rank. Always 0 if not running in an MPI context
+        self.mpi_size: uint
+            Number of ranks in COMM_WORLD. 1 if not running in an MPI context
+        self.__ranks_on_socket : uint
+            Number of MPI ranks on a given CPU socket
+        self.__socket_master_rank : uint
+            Master MPI rank for a given CPU chip / socket
+        self.__compute_jobs : array-like
+            List of positions in the HDF5 dataset that need to be computed.
+            This may not be a continuous list of numbers if multiple MPI
+            workers had previously started computing and were interrupted.
+        self.__start_pos : uint
+            The index within self.__compute_jobs that a particular MPI rank /
+            worker needs to start computing from.
+        self.__rank_end_pos : uint
+            The index within self.__compute_jobs that a particular MPI rank /
+            worker needs to start computing till.
+        self.__end_pos : uint
+            The index within self.__compute_jobs that a particular MPI rank /
+            worker needs to start computing till for the current batch of
+            positions.
+        self.__pixels_in_batch : array-like
+            The positions being computed on by the current compute worker
         """
 
         if h5_main.file.mode != 'r+':
@@ -111,6 +187,14 @@ class Process(object):
         if not check_if_main(h5_main, verbose=verbose and self.mpi_rank == 0):
             raise ValueError('Provided dataset is not a "Main" dataset with necessary ancillary datasets')
 
+        process_name = validate_single_string_arg(process_name, 'process_name')
+
+        if parms_dict is None:
+            parms_dict = {}
+        else:
+            if not isinstance(parms_dict, dict):
+                raise TypeError("Expected 'parms_dict' of type: dict")
+
         if MPI is not None:
             MPI.COMM_WORLD.barrier()
         # Not sure if we need a barrier here.
@@ -126,6 +210,7 @@ class Process(object):
 
         # Saving these as properties of the object:
         self.verbose = verbose
+        self.__lazy = lazy
         self._cores = None
         self.__ranks_on_socket = 1
         self.__socket_master_rank = 0
@@ -137,6 +222,7 @@ class Process(object):
         self.__rank_end_pos = None
         self.__end_pos = None
         self.__pixels_in_batch = None
+        self.__compute_jobs = None
 
         # Determining the max size of the data that can be put into memory
         # all ranks go through this and they need to have this value any
@@ -146,8 +232,8 @@ class Process(object):
             print('Finished collecting info on memory and workers')
         self.duplicate_h5_groups = []
         self.partial_h5_groups = []
-        self.process_name = None  # Reset this in the extended classes
-        self.parms_dict = None
+        self.process_name = process_name  # Reset this in the extended classes
+        self.parms_dict = parms_dict
 
         """
         The name of the HDF5 dataset that should be present to signify which positions have already been computed
@@ -178,23 +264,30 @@ class Process(object):
             print('Consider calling test() to check results before calling compute() which computes on the entire'
                   ' dataset and writes back to the HDF5 file')
 
-        # DON'T check for duplicates since parms_dict has not yet been initialized.
-        # Sub classes will check by themselves if they are interested.
+        self.duplicate_h5_groups, self.partial_h5_groups = self._check_for_duplicates()
 
     def __assign_job_indices(self):
         """
         Sets the start and end indices for each MPI rank
         """
         # First figure out what positions need to be computed
-        self._compute_jobs = np.where(self._h5_status_dset[()] == 0)[0]
+        self.__compute_jobs = np.where(self._h5_status_dset[()] == 0)[0]
         if self.verbose and self.mpi_rank == 0:
-            print('Among the {} positions in this dataset, the following positions need to be computed: {}'
-                  '.'.format(self.h5_main.shape[0], self._compute_jobs))
+            if len(self.__compute_jobs) > 100:
+                print('Among the {} positions in this dataset, {} positions '
+                      'need to be computed'
+                      '.'.format(self.h5_main.shape[0],
+                                 len(self.__compute_jobs)))
+            else:
+                print('Among the {} positions in this dataset, the following '
+                      'positions need to be computed: {}'
+                      '.'.format(self.h5_main.shape[0], self.__compute_jobs))
 
-        pos_per_rank = self._compute_jobs.size // self.mpi_size  # integer division
+        # integer division
+        pos_per_rank = self.__compute_jobs.size // self.mpi_size
         if self.verbose and self.mpi_rank == 0:
             print('Each rank is required to work on {} of the {} (remaining) positions in this dataset'
-                  '.'.format(pos_per_rank, self._compute_jobs.size))
+                  '.'.format(pos_per_rank, self.__compute_jobs.size))
 
         # The start and end indices now correspond to the indices in the incomplete jobs rather than the h5 dataset
         self.__start_pos = self.mpi_rank * pos_per_rank
@@ -202,7 +295,7 @@ class Process(object):
         self.__end_pos = int(min(self.__rank_end_pos, self.__start_pos + self._max_pos_per_read))
         if self.mpi_rank == self.mpi_size - 1:
             # Force the last rank to go to the end of the dataset
-            self.__rank_end_pos = self._compute_jobs.size
+            self.__rank_end_pos = self.__compute_jobs.size
 
         if self.verbose:
             print('Rank {} will read positions {} to {} of {}'.format(self.mpi_rank, self.__start_pos,
@@ -262,92 +355,113 @@ class Process(object):
         Returns
         -------
         duplicate_h5_groups : list of h5py.Group objects
-            List of groups satisfying the above conditions
+            List of groups satisfying the above conditions with completely computed results
+        partial_h5_groups : list of h5py.Group objects
+            List of groups satisfying the above conditions with partially computed results
         """
         if self.verbose and self.mpi_rank == 0:
             print('Checking for duplicates:')
 
         # This list will contain completed runs only
-        duplicate_h5_groups = check_for_old(self.h5_main, self.process_name,
-                                            new_parms=self.parms_dict,
-                                            verbose=self.verbose and self.mpi_rank==0)
+        existing = check_for_old(self.h5_main, self.process_name,
+                                 new_parms=self.parms_dict,
+                                 verbose=self.verbose and self.mpi_rank == 0)
+
         partial_h5_groups = []
+        duplicate_h5_groups = []
 
         # First figure out which ones are partially completed:
-        if len(duplicate_h5_groups) > 0:
-            for index, curr_group in enumerate(duplicate_h5_groups):
-                """
-                Earlier, we only checked the 'last_pixel' but to be rigorous we should check self._status_dset_name
-                The last_pixel attribute check may be deprecated in the future.
-                Note that legacy computations did not have this dataset. We can add to partially computed datasets
-                """
-                # Case 1: Modern book-keeping dataset available:
-                if self._status_dset_name in curr_group.keys():
+        while len(existing) > 0:
+            curr_group = existing.pop(0)
+            """
+            Earlier, we only checked the 'last_pixel' but to be rigorous we 
+            should check self._status_dset_name
+            The last_pixel attribute check may be deprecated in the future.
+            Note that legacy computations did not have this dataset. We can add 
+            to partially computed datasets
+            """
 
-                    status_dset = curr_group[self._status_dset_name]
+            # Case 1: Modern book-keeping dataset available:
+            if self._status_dset_name in curr_group.keys():
 
-                    if not isinstance(status_dset, h5py.Dataset):
-                        # We should not come here if things were implemented correctly
-                        if self.mpi_rank == 0:
-                            print('Results group: {} contained an object named: {} that should have been a dataset'
-                                  '.'.format(curr_group, self._status_dset_name))
+                status_dset = curr_group[self._status_dset_name]
 
-                    if self.h5_main.shape[0] != status_dset.shape[0] or len(status_dset.shape) > 1 or \
-                            status_dset.dtype != np.uint8:
-                        if self.mpi_rank == 0:
-                            print('Status dataset: {} was not of the expected shape or datatype'.format(status_dset))
-
-                    # ##### ACTUAL COMPLETENESS TEST HERE #########
-
-                    completed_positions = np.sum(status_dset[()])
-
-                    if self.verbose and self.mpi_rank == 0:
-                        print('{} has results that are {} % complete'
-                              '.'.format(status_dset.name,
-                                         int(100 * completed_positions / self.h5_main.shape[0])))
-
-                    # Case 1.A: Incomplete computation?
-                    if completed_positions < self.h5_main.shape[0]:
-                        # If there are pixels uncompleted
-                        # remove from duplicates and move to partial
-                        if self.verbose and self.mpi_rank == 0:
-                            print('moving {} to partial'.format(curr_group.name))
-                        partial_h5_groups.append(duplicate_h5_groups.pop(index))
-                        # Let's write the legacy attribute for safety
-                        curr_group.attrs['last_pixel'] = self.h5_main.shape[0]
-                        # No further checks necessary
-                        continue
-
-                    # Case 1.B: Complete computation:
-                    if self.verbose and self.mpi_rank == 0:
-                        print('Leaving {} in duplicate groups'.format(curr_group.name))
-                    continue
-
-                # Case 2: Even the legacy book-keeping is absent:
-                elif 'last_pixel' not in curr_group.attrs.keys():
+                if not isinstance(status_dset, h5py.Dataset):
+                    # We should not come here if things were implemented correctly
                     if self.mpi_rank == 0:
-                        # Should not be coming here at all
-                        print('Group: {} had neither the status HDF5 dataset or the legacy attribute: "last_pixel"'
-                              '.'.format(curr_group))
-                    # Not sure what to do with such groups. Don't consider them
-                    duplicate_h5_groups.pop(index)
+                        print('Results group: {} contained an object named: {} that should have been a dataset'
+                              '.'.format(curr_group, self._status_dset_name))
                     continue
 
-                # Case 3.A: Only the legacy book-keeping is available:
-                elif curr_group.attrs['last_pixel'] < self.h5_main.shape[0]:
-                    # Should we create the dataset here, to make the group future-proof?
+                if self.h5_main.shape[0] != status_dset.shape[0] or len(status_dset.shape) > 1 or \
+                        status_dset.dtype != np.uint8:
+                    if self.mpi_rank == 0:
+                        print('Status dataset: {} was not of the expected shape or datatype'.format(status_dset))
+                    continue
+
+                # ##### ACTUAL COMPLETENESS TEST HERE #########
+
+                completed_positions = np.sum(status_dset[()])
+
+                if self.verbose and self.mpi_rank == 0:
+                    print('{} has results that are {} % complete'
+                          '.'.format(status_dset.name,
+                                     int(100 * completed_positions / self.h5_main.shape[0])))
+
+                # Case 1.A: Incomplete computation?
+                if completed_positions < self.h5_main.shape[0]:
+                    # If there are pixels uncompleted
                     # remove from duplicates and move to partial
+                    if self.verbose and self.mpi_rank == 0:
+                        print('moving {} to partial'.format(curr_group.name))
+                    partial_h5_groups.append(curr_group)
+                    # Let's write the legacy attribute for safety
+                    curr_group.attrs['last_pixel'] = self.h5_main.shape[0]
+                    # No further checks necessary
+                    continue
+
+                # Case 1.B: Complete computation:
+                if self.verbose and self.mpi_rank == 0:
+                    print('Moving {} to duplicate groups'.format(curr_group.name))
+                duplicate_h5_groups.append(curr_group)
+                continue
+
+            # Case 2: Even the legacy book-keeping is absent:
+            elif 'last_pixel' not in curr_group.attrs.keys():
+                if self.mpi_rank == 0:
+                    # Should not be coming here at all
+                    print('Group: {} had neither the status HDF5 dataset or the legacy attribute: "last_pixel"'
+                          '.'.format(curr_group))
+                # Not sure what to do with such groups. Don't consider them
+                continue
+
+            # Case 3: Only the legacy book-keeping is available:
+            else:
+                last_pixel = curr_group.attrs['last_pixel']
+
+                # Creating status dataset for forward compatibility:
+                self._h5_status_dset = curr_group.create_dataset(
+                    self._status_dset_name, dtype=np.uint8,
+                    shape=(self.h5_main.shape[0],))
+                if last_pixel > 0:
+                    self._h5_status_dset[:last_pixel] = 1
+
+                # Case 3.A: Partial
+                if last_pixel < self.h5_main.shape[0]:
+                    # move to partial
                     if self.verbose and self.mpi_rank == 0:
                         print('moving {} to partial since computation was {} % complete'
                               '.'.format(curr_group.name,
                                          int(100 * curr_group.attrs['last_pixel'] / self.h5_main.shape[0])))
-                    partial_h5_groups.append(duplicate_h5_groups.pop(index))
-
+                    partial_h5_groups.append(curr_group)
                     continue
 
-                # Case 4: legacy book-keeping shows that computation is complete:
-                if self.verbose and self.mpi_rank == 0:
-                    print('Leaving {} in duplicate groups'.format(curr_group.name))
+                # Case 3.B: complete:
+                else:
+                    if self.verbose and self.mpi_rank == 0:
+                        print('Moving {} to duplicate groups'.format(curr_group.name))
+                    duplicate_h5_groups.append(curr_group)
+                    continue
 
         if len(duplicate_h5_groups) > 0 and self.mpi_rank == 0:
             print('\nNote: ' + self.process_name + ' has already been performed with the same parameters before. '
@@ -384,7 +498,8 @@ class Process(object):
             if h5_partial_group not in self.partial_h5_groups:
                 raise ValueError('Provided group does not appear to be in the list of discovered groups')
 
-        self.parms_dict = get_attributes(h5_partial_group)
+        # Unnecessary since this will be defined at init
+        # self.parms_dict = get_attributes(h5_partial_group)
 
         self.h5_results_grp = h5_partial_group
 
@@ -567,7 +682,7 @@ class Process(object):
             self.__end_pos = int(min(self.__rank_end_pos, self.__start_pos + self._max_pos_per_read))
 
             # DON'T DIRECTLY apply the start and end indices anymore to the h5 dataset. Find out what it means first
-            self.__pixels_in_batch = self._compute_jobs[self.__start_pos: self.__end_pos]
+            self.__pixels_in_batch = self.__compute_jobs[self.__start_pos: self.__end_pos]
 
             if self.verbose:
                 print('Rank {} will read positions: {}'.format(self.mpi_rank, self.__pixels_in_batch))
@@ -581,9 +696,16 @@ class Process(object):
                           '.'.format(self.mpi_rank,
                                      format_size(get_available_memory()),
                                      tot_workers,
-                                     bytes_this_read * tot_workers))
+                                     format_size(bytes_this_read * tot_workers)
+                                     ))
 
-            self.data = self.h5_main[self.__pixels_in_batch, :]
+            # Reading as Dask array to minimize memory copies when restructuring in child classes
+            if self.__lazy:
+                main_dset = lazy_load_array(self.h5_main)
+            else:
+                main_dset = self.h5_main
+
+            self.data = main_dset[self.__pixels_in_batch, :]
             # DON'T update the start position
 
         else:
@@ -742,8 +864,9 @@ class Process(object):
             if len(self.duplicate_h5_groups) > 0:
                 if self.mpi_rank == 0:
                     print('Returned previously computed results at ' + self.duplicate_h5_groups[-1].name)
+                self.h5_results_grp = self.duplicate_h5_groups[-1]
                 return self.duplicate_h5_groups[-1]
-            elif len(self.partial_h5_groups) > 0:
+            elif len(self.partial_h5_groups) > 0 and self.h5_results_grp is None:
                 if self.mpi_rank == 0:
                     print('Resuming computation in group: ' + self.partial_h5_groups[-1].name)
                 self.use_partial_computation()
@@ -870,39 +993,3 @@ class Process(object):
             self.h5_results_grp.attrs['last_pixel'] = self.h5_main.shape[0]
 
         return self.h5_results_grp
-
-
-def parallel_compute(data, func, cores=1, lengthy_computation=False, func_args=None, func_kwargs=None, verbose=False):
-    """
-    Computes the provided function using multiple cores using the joblib library
-
-    Parameters
-    ----------
-    data : numpy.ndarray
-        Data to map function to. Function will be mapped to the first axis of data
-    func : callable
-        Function to map to data
-    cores : uint, optional
-        Number of logical cores to use to compute
-        Default - 1 (serial computation)
-    lengthy_computation : bool, optional
-        Whether or not each computation is expected to take substantial time.
-        Sometimes the time for adding more cores can outweigh the time per core
-        Default - False
-    func_args : list, optional
-        arguments to be passed to the function
-    func_kwargs : dict, optional
-        keyword arguments to be passed onto function
-    verbose : bool, optional. default = False
-        Whether or not to print statements that aid in debugging
-    Returns
-    -------
-    results : list
-        List of computational results
-    """
-    from warnings import warn
-    warn('Please call this function either as pyUSID.processing.comp_utils.parallel_compute() or simply as '
-         'pyUSID.parallel_compute() instead in the future', FutureWarning)
-    from .comp_utils import parallel_compute
-    return parallel_compute(data, func, cores=cores, lengthy_computation=lengthy_computation, func_args=func_args,
-                            func_kwargs=func_kwargs, verbose=verbose)
