@@ -87,7 +87,8 @@ def capture_stdout():
 
 
 def validate_aux_dset_pair(test_class, h5_group, h5_inds, h5_vals, dim_names, dim_units, inds_matrix,
-                           vals_matrix=None, base_name=None, h5_main=None, is_spectral=True):
+                           vals_matrix=None, base_name=None, h5_main=None, is_spectral=True,
+                           slow_to_fast=False):
     if vals_matrix is None:
         vals_matrix = inds_matrix
     if base_name is None:
@@ -98,6 +99,18 @@ def validate_aux_dset_pair(test_class, h5_group, h5_inds, h5_vals, dim_names, di
     else:
         test_class.assertIsInstance(base_name, (str, unicode))
 
+    if not slow_to_fast:
+        # Sending in to Fast to Slow but what comes out is slow to fast
+        func = np.flipud if is_spectral else np.fliplr
+
+        print(inds_matrix)
+
+        vals_matrix = func(vals_matrix)
+        inds_matrix = func(inds_matrix)
+
+        dim_names = dim_names[::-1]
+        dim_units = dim_units[::-1]
+
     for h5_dset, exp_dtype, exp_name, ref_data in zip([h5_inds, h5_vals],
                                                       [INDICES_DTYPE, VALUES_DTYPE],
                                                       [base_name + '_Indices', base_name + '_Values'],
@@ -136,7 +149,7 @@ def make_sparse_sampling_file():
 
     h5_main = None
 
-    with h5py.File(sparse_sampling_path) as h5_f:
+    with h5py.File(sparse_sampling_path, mode='w') as h5_f:
         h5_meas_grp = h5_f.create_group('Measurement_000')
 
         freq_pts = 3
@@ -201,7 +214,7 @@ def make_incomplete_measurement_file():
     if os.path.exists(incomplete_measurement_path):
         os.remove(incomplete_measurement_path)
 
-    with h5py.File(incomplete_measurement_path) as h5_f:
+    with h5py.File(incomplete_measurement_path, mode='w') as h5_f:
         h5_meas_grp = h5_f.create_group('Measurement_000')
 
         freq_pts = 3
@@ -274,7 +287,7 @@ def make_relaxation_file():
     if os.path.exists(relaxation_path):
         os.remove(relaxation_path)
 
-    with h5py.File(relaxation_path) as h5_f:
+    with h5py.File(relaxation_path, mode='w') as h5_f:
         h5_meas_grp = h5_f.create_group('Measurement_000')
 
         num_rows = 2
@@ -359,7 +372,7 @@ def make_beps_file(rev_spec=False):
     if os.path.exists(std_beps_path):
         os.remove(std_beps_path)
 
-    with h5py.File(std_beps_path) as h5_f:
+    with h5py.File(std_beps_path, mode='w') as h5_f:
 
         h5_raw_grp = h5_f.create_group('Raw_Measurement')
         write_safe_attrs(h5_raw_grp, {'att_1': 'string_val', 'att_2': 1.2345, 'att_3': [1, 2, 3, 4]})
@@ -375,6 +388,7 @@ def make_beps_file(rev_spec=False):
         source_dset_name = 'source_main'
         tool_name = 'Fitter'
 
+        # Per USID, dimensions are arranged from fastest to slowest
         source_pos_data = np.vstack((np.tile(np.arange(num_cols), num_rows),
                                      np.repeat(np.arange(num_rows), num_cols))).T
         pos_attrs = {'units': ['nm', 'um'], 'labels': ['X', 'Y']}
@@ -384,7 +398,15 @@ def make_beps_file(rev_spec=False):
         write_string_list_as_attr(h5_pos_inds, pos_attrs)
 
         # make the values more interesting:
-        source_pos_data = np.vstack((source_pos_data[:, 0] * 50, source_pos_data[:, 1] * 1.25)).T
+        cols_offset = -750
+        cols_step = 50
+        rows_offset = 2
+        rows_step = 1.25
+        source_pos_data = np.vstack((cols_offset + source_pos_data[:, 0] * cols_step,
+                                     rows_offset + source_pos_data[:, 1] * rows_step)).T
+
+        _ = h5_raw_grp.create_dataset('X', data=cols_offset + cols_step * np.arange(num_cols))
+        _ = h5_raw_grp.create_dataset('Y', data=rows_offset + rows_step * np.arange(num_rows))
 
         h5_pos_vals = h5_raw_grp.create_dataset('Position_Values', data=source_pos_data, dtype=np.float32)
         write_aux_reg_ref(h5_pos_vals, pos_attrs['labels'], is_spec=False)
@@ -406,11 +428,18 @@ def make_beps_file(rev_spec=False):
         write_string_list_as_attr(h5_source_spec_inds, source_spec_attrs)
 
         # make spectroscopic axis interesting as well
+        bias_amp = 2.5
+        bias_period = np.pi
+        bias_vec = bias_amp * np.sin(np.linspace(0, bias_period, num_cycle_pts, endpoint=False))
+
+        _ = h5_raw_grp.create_dataset('Bias', data=bias_vec)
+        _ = h5_raw_grp.create_dataset('Cycle', data=np.arange(num_cycles))
+
         if rev_spec:
             source_spec_data = np.vstack((np.repeat(np.arange(num_cycles), num_cycle_pts),
-                                          np.tile(2.5 * np.sin(np.linspace(0, np.pi, num_cycle_pts, endpoint=False)), num_cycles)))
+                                          np.tile(bias_vec, num_cycles)))
         else:
-            source_spec_data = np.vstack((np.tile(2.5 * np.sin(np.linspace(0, np.pi, num_cycle_pts, endpoint=False)), num_cycles),
+            source_spec_data = np.vstack((np.tile(bias_vec, num_cycles),
                                           np.repeat(np.arange(num_cycles), num_cycle_pts)))
 
         h5_source_spec_vals = h5_raw_grp.create_dataset('Spectroscopic_Values', data=source_spec_data,
@@ -418,7 +447,16 @@ def make_beps_file(rev_spec=False):
         write_aux_reg_ref(h5_source_spec_vals, source_spec_attrs['labels'], is_spec=True)
         write_string_list_as_attr(h5_source_spec_vals, source_spec_attrs)
 
-        source_main_data = np.random.rand(num_rows * num_cols, num_cycle_pts * num_cycles)
+        main_nd = np.random.rand(num_rows, num_cols, num_cycles, num_cycle_pts)
+        h5_nd_main = h5_raw_grp.create_dataset('n_dim_form', data=main_nd)
+        write_string_list_as_attr(h5_nd_main, {'dims': ['Y', 'X', 'Cycle', 'Bias']})
+
+        if rev_spec:
+            # This simulates things like BEPS where Field should actually be varied slower but is varied faster during acquisition
+            main_nd = main_nd.transpose(0, 1, 3, 2)
+
+        source_main_data = main_nd.reshape(num_rows * num_cols, num_cycle_pts * num_cycles)
+        # source_main_data = np.random.rand(num_rows * num_cols, num_cycle_pts * num_cycles)
         h5_source_main = h5_raw_grp.create_dataset(source_dset_name, data=source_main_data)
         write_safe_attrs(h5_source_main, {'units': 'A', 'quantity': 'Current'})
         write_main_reg_refs(h5_source_main, {'even_rows': (slice(0, None, 2), slice(None)),
@@ -455,7 +493,24 @@ def make_beps_file(rev_spec=False):
         write_aux_reg_ref(h5_results_1_spec_vals, results_spec_attrs['labels'], is_spec=True)
         write_string_list_as_attr(h5_results_1_spec_vals, results_spec_attrs)
 
-        results_1_main_data = np.random.rand(num_rows * num_cols, num_cycle_pts * num_cycles)
+        # Let this be a compound dataset:
+        struc_dtype = np.dtype({'names': ['r', 'g', 'b'],
+                                'formats': [np.float32, np.float16,
+                                            np.float64]})
+
+        num_elems = (num_rows, num_cols, num_cycles, num_cycle_pts)
+        results_1_nd = np.zeros(shape=num_elems, dtype=struc_dtype)
+        for name_ind, name in enumerate(struc_dtype.names):
+            results_1_nd[name] = np.random.random(size=num_elems)
+
+        h5_results_1_nd = h5_results_grp_1.create_dataset('n_dim_form',
+                                                          data=results_1_nd)
+        write_string_list_as_attr(h5_results_1_nd,
+                                  {'dims': ['Y', 'X', 'Cycle', 'Bias']})
+
+        results_1_main_data = results_1_nd.reshape(num_rows * num_cols,
+                                                   num_cycle_pts * num_cycles)
+
         h5_results_1_main = h5_results_grp_1.create_dataset('results_main', data=results_1_main_data)
         write_safe_attrs(h5_results_1_main, {'units': 'pF', 'quantity': 'Capacitance'})
 
@@ -470,7 +525,17 @@ def make_beps_file(rev_spec=False):
                                         {'att_1': 'other_string_val', 'att_2': 5.4321, 'att_3': [4, 1, 3]})
         write_string_list_as_attr(h5_results_grp_2, {'att_4': ['s', 'str_2', 'str_3']})
 
-        results_2_main_data = np.random.rand(num_rows * num_cols, num_cycle_pts * num_cycles)
+        # Let these results be a complex typed dataset:
+        results_2_nd = np.random.random(size=num_elems) + \
+                       1j * np.random.random(size=num_elems)
+
+        h5_results_2_nd = h5_results_grp_2.create_dataset('n_dim_form',
+                                                          data=results_2_nd)
+        write_string_list_as_attr(h5_results_2_nd,
+                                  {'dims': ['Y', 'X', 'Cycle', 'Bias']})
+
+        results_2_main_data = results_2_nd.reshape(num_rows * num_cols,
+                                                   num_cycle_pts * num_cycles)
         h5_results_2_main = h5_results_grp_2.create_dataset('results_main', data=results_2_main_data)
         write_safe_attrs(h5_results_2_main, {'units': 'pF', 'quantity': 'Capacitance'})
 
