@@ -15,7 +15,7 @@ import shutil
 sys.path.append("../../pyUSID/")
 from pyUSID.io import hdf_utils, write_utils, USIDataset
 
-from tests.processing import data_utils
+from .. import data_utils
 
 if sys.version_info.major == 3:
     unicode = str
@@ -40,7 +40,7 @@ class TestSimple(unittest.TestCase):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(11, 7)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_dest = h5_f.create_dataset('Target', data=data)
             source_ref = h5_dset_source.regionref[0:-1:2]
@@ -79,7 +79,7 @@ class TestCheckIfMain(TestSimple):
     def test_anc_not_dsets(self):
         temp_path = 'test.h5'
         data_utils.delete_existing_file(temp_path)
-        with h5py.File(temp_path) as h5_f:
+        with h5py.File(temp_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=np.random.rand(2, 3))
             for anc_dset_name in ['Position_Indices', 'Position_Values',
                                   'Spectroscopic_Indices', 'Spectroscopic_Values']:
@@ -90,7 +90,7 @@ class TestCheckIfMain(TestSimple):
     def test_missing_str_attrs(self):
         temp_path = 'test.h5'
         data_utils.delete_existing_file(temp_path)
-        with h5py.File(temp_path) as h5_f:
+        with h5py.File(temp_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=np.random.rand(2, 3))
             for anc_dset_name in ['Position_Indices', 'Position_Values',
                                   'Spectroscopic_Indices', 'Spectroscopic_Values']:
@@ -101,7 +101,7 @@ class TestCheckIfMain(TestSimple):
     def test_invalid_str_attrs(self):
         temp_path = 'test.h5'
         data_utils.delete_existing_file(temp_path)
-        with h5py.File(temp_path) as h5_f:
+        with h5py.File(temp_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=np.random.rand(2, 3))
             h5_dset.attrs['quantity'] = [1, 2, 3]
             h5_dset.attrs['units'] = 4.1234
@@ -114,7 +114,7 @@ class TestCheckIfMain(TestSimple):
     def test_anc_shapes_not_matching(self):
         temp_path = 'test.h5'
         data_utils.delete_existing_file(temp_path)
-        with h5py.File(temp_path) as h5_f:
+        with h5py.File(temp_path, mode='w') as h5_f:
             h5_main = h5_f.create_dataset('Main', data=np.random.rand(2, 3))
             h5_pos_ind = h5_f.create_dataset('Pos_Inds', data=np.random.rand(2, 1))
             h5_spec_ind = h5_f.create_dataset('Spec_Inds', data=np.random.rand(1, 5))
@@ -124,7 +124,7 @@ class TestCheckIfMain(TestSimple):
                 h5_main.attrs[anc_dset_name] = h5_pos_ind.ref
             for anc_dset_name in ['Spectroscopic_Indices', 'Spectroscopic_Values']:
                 h5_main.attrs[anc_dset_name] = h5_spec_ind.ref
-            self.assertFalse(hdf_utils.check_if_main(h5_main, verbose=True))
+            self.assertFalse(hdf_utils.check_if_main(h5_main, verbose=False))
         os.remove(temp_path)
 
 
@@ -170,52 +170,55 @@ class TestGetAllMain(TestSimple):
 
 class TestWriteIndValDsets(TestSimple):
 
-    def test_legal_bare_minimum_pos(self):
+    def base_bare_minimum_inputs(self, slow_to_fast, is_spectral):
         num_cols = 3
         num_rows = 2
         sizes = [num_cols, num_rows]
         dim_names = ['X', 'Y']
         dim_units = ['nm', 'um']
 
-        descriptor = []
-        for length, name, units in zip(sizes, dim_names, dim_units):
-            descriptor.append(write_utils.Dimension(name, units, np.arange(length)))
-
-        pos_data = np.vstack((np.tile(np.arange(num_cols), num_rows),
-                              np.repeat(np.arange(num_rows), num_cols))).T
-        file_path = 'test_write_ind_val_dsets.h5'
-        data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path, mode='w') as h5_f:
-            h5_inds, h5_vals = hdf_utils.write_ind_val_dsets(h5_f, descriptor, is_spectral=False)
-
-            data_utils.validate_aux_dset_pair(self, h5_f, h5_inds, h5_vals, dim_names, dim_units, pos_data,
-                                              is_spectral=False)
-
-        os.remove(file_path)
-
-    def test_legal_bare_minimum_spec(self):
-        num_cols = 3
-        num_rows = 2
-        sizes = [num_cols, num_rows]
-        dim_names = ['X', 'Y']
-        dim_units = ['nm', 'um']
+        if slow_to_fast:
+            dim_names = dim_names[::-1]
+            dim_units = dim_units[::-1]
+            sizes = sizes[::-1]
 
         descriptor = []
         for length, name, units in zip(sizes, dim_names, dim_units):
             descriptor.append(write_utils.Dimension(name, units, np.arange(length)))
 
-        spec_data = np.vstack((np.tile(np.arange(num_cols), num_rows),
-                              np.repeat(np.arange(num_rows), num_cols)))
+        inds_data = np.vstack((np.tile(np.arange(num_cols), num_rows),
+                              np.repeat(np.arange(num_rows), num_cols)))\
+
+        if not is_spectral:
+            inds_data = inds_data.T
+
+        if slow_to_fast:
+            func = np.flipud if is_spectral else np.fliplr
+            inds_data = func(inds_data)
+
         file_path = 'test_write_ind_val_dsets.h5'
         data_utils.delete_existing_file(file_path)
         with h5py.File(file_path, mode='w') as h5_f:
-            h5_group = h5_f.create_group("Blah")
-            h5_inds, h5_vals = hdf_utils.write_ind_val_dsets(h5_group, descriptor, is_spectral=True)
+            h5_inds, h5_vals = hdf_utils.write_ind_val_dsets(h5_f, descriptor, is_spectral=is_spectral,
+                                                             slow_to_fast=slow_to_fast)
+
+            data_utils.validate_aux_dset_pair(self, h5_f, h5_inds, h5_vals, dim_names, dim_units, inds_data,
+                                              is_spectral=is_spectral, slow_to_fast=slow_to_fast)
 
-            data_utils.validate_aux_dset_pair(self, h5_group, h5_inds, h5_vals, dim_names, dim_units, spec_data,
-                                              is_spectral=True)
         os.remove(file_path)
 
+    def test_legal_bare_minimum_pos_f2s(self):
+        self.base_bare_minimum_inputs(False, False)
+
+    def test_legal_bare_minimum_pos_s2f(self):
+        self.base_bare_minimum_inputs(True, False)
+
+    def test_legal_bare_minimum_spec_f2s(self):
+        self.base_bare_minimum_inputs(False, True)
+
+    def test_legal_bare_minimum_spec_s2f(self):
+        self.base_bare_minimum_inputs(True, True)
+
     def test_legal_override_steps_offsets_base_name(self):
         num_cols = 2
         num_rows = 3
@@ -232,6 +235,7 @@ class TestWriteIndValDsets(TestSimple):
             descriptor.append(write_utils.Dimension(name, units, initial + step * np.arange(length)))
 
         new_base_name = 'Overriden'
+        # Sending in Fast to Slow but what comes out is slow to fast
         spec_inds = np.vstack((np.tile(np.arange(num_cols), num_rows),
                               np.repeat(np.arange(num_rows), num_cols)))
         spec_vals = np.vstack((np.tile(np.arange(num_cols), num_rows) * col_step + col_initial,
@@ -242,9 +246,9 @@ class TestWriteIndValDsets(TestSimple):
         with h5py.File(file_path, mode='w') as h5_f:
             h5_group = h5_f.create_group("Blah")
             h5_inds, h5_vals = hdf_utils.write_ind_val_dsets(h5_group, descriptor, is_spectral=True,
-                                                             base_name=new_base_name)
+                                                             base_name=new_base_name, slow_to_fast=False)
             data_utils.validate_aux_dset_pair(self, h5_group, h5_inds, h5_vals, dim_names, dim_units, spec_inds,
-                                              vals_matrix=spec_vals, base_name=new_base_name, is_spectral=True)
+                                          vals_matrix=spec_vals, base_name=new_base_name, is_spectral=True, slow_to_fast=False)
         os.remove(file_path)
 
     def test_illegal(self):
@@ -274,7 +278,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
         shutil.copy(data_utils.std_beps_path, duplicate_path)
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='r+') as h5_f:
             h5_spec_inds_orig = h5_f['/Raw_Measurement/Spectroscopic_Indices']
             h5_spec_vals_orig = h5_f['/Raw_Measurement/Spectroscopic_Values']
             new_base_name = 'Blah'
@@ -310,7 +314,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             h5_spec_inds_orig, h5_spec_vals_orig = hdf_utils.write_ind_val_dsets(h5_f,
                                                                                  write_utils.Dimension('Bias', 'V', 10),
@@ -345,7 +349,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             dims = [write_utils.Dimension('X', 'nm', np.linspace(300, 350, 5)),
                     write_utils.Dimension('Y', 'um', [-2, 4, 10]),
@@ -384,7 +388,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             dims = [write_utils.Dimension('Freq', 'Hz', np.linspace(300, 350, 5)),
                     write_utils.Dimension('Bias', 'V', [-2, 4, 10]),
@@ -424,7 +428,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             dims = [write_utils.Dimension('Freq', 'Hz', np.linspace(300, 350, 5)),
                     write_utils.Dimension('Bias', 'V', [-2, 4, 10]),
@@ -464,7 +468,7 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             dims = [write_utils.Dimension('Freq', 'Hz', np.linspace(300, 350, 5)),
                     write_utils.Dimension('Bias', 'V', [-2, 4, 10]),
@@ -504,13 +508,14 @@ class TestWriteReducedAncDsets(TestSimple):
         duplicate_path = 'copy_test_hdf_utils.h5'
         data_utils.delete_existing_file(duplicate_path)
 
-        with h5py.File(duplicate_path) as h5_f:
+        with h5py.File(duplicate_path, mode='w') as h5_f:
 
             dims = [write_utils.Dimension('Freq', 'Hz', np.linspace(300, 350, 5)),
                     write_utils.Dimension('Bias', 'V', [-2, 4, 10]),
                     write_utils.Dimension('Cycle', 'a.u.', 2)]
 
-            h5_spec_inds_orig, h5_spec_vals_orig = hdf_utils.write_ind_val_dsets(h5_f, dims, is_spectral=True)
+            h5_spec_inds_orig, h5_spec_vals_orig = hdf_utils.write_ind_val_dsets(h5_f, dims, is_spectral=True,
+                                                                                 slow_to_fast=False)
             new_base_name = 'Blah'
             h5_spec_inds_new, h5_spec_vals_new = hdf_utils.write_reduced_anc_dsets(h5_f, h5_spec_inds_orig,
                                                                                    h5_spec_vals_orig,
@@ -523,6 +528,13 @@ class TestWriteReducedAncDsets(TestSimple):
                                   np.repeat(np.arange(2), 5)))
             ref_inds = np.vstack((np.tile(np.arange(5, dtype=np.uint16), 2),
                                   np.repeat(np.arange(2, dtype=np.uint16), 5)))
+
+            # Sending in Fast to Slow but what comes out is slow to fast
+            ref_inds = np.flipud(ref_inds)
+            ref_vals = np.flipud(ref_vals)
+            dim_names = dim_names[::-1]
+            dim_units = dim_units[::-1]
+
             for h5_dset, exp_dtype, exp_name, ref_data in zip([h5_spec_inds_new, h5_spec_vals_new],
                                                               [h5_spec_inds_orig.dtype, h5_spec_vals_orig.dtype],
                                                               [new_base_name + '_Indices', new_base_name + '_Values'],
@@ -666,7 +678,7 @@ class TestCheckForOld(TestSimple):
             h5_main = h5_f['/Raw_Measurement/source_main']
             attrs = {'units': ['V'], 'labels': ['Bias']}
             dset_name = 'Spectroscopic_Indices'
-            groups = hdf_utils.check_for_old(h5_main, 'Fitter', new_parms=attrs, target_dset=dset_name, verbose=True)
+            groups = hdf_utils.check_for_old(h5_main, 'Fitter', new_parms=attrs, target_dset=dset_name, verbose=False)
             groups = set(groups)
             self.assertEqual(groups, set([h5_f['/Raw_Measurement/source_main-Fitter_000/'],
                                           h5_f['/Raw_Measurement/source_main-Fitter_001/']]))
@@ -727,7 +739,7 @@ class TestCreateIndexedGroup(TestSimple):
     def test_first_group(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_group = hdf_utils.create_indexed_group(h5_f, 'Hello')
             self.assertIsInstance(h5_group, h5py.Group)
             self.assertEqual(h5_group.name, '/Hello_000')
@@ -744,7 +756,7 @@ class TestCreateIndexedGroup(TestSimple):
     def test_second(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_group_1 = hdf_utils.create_indexed_group(h5_f, 'Hello')
             self.assertIsInstance(h5_group_1, h5py.Group)
             self.assertEqual(h5_group_1.name, '/Hello_000')
@@ -761,7 +773,7 @@ class TestCreateIndexedGroup(TestSimple):
     def test_w_suffix_(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_group = hdf_utils.create_indexed_group(h5_f, 'Hello_')
             self.assertIsInstance(h5_group, h5py.Group)
             self.assertEqual(h5_group.name, '/Hello_000')
@@ -772,7 +784,7 @@ class TestCreateIndexedGroup(TestSimple):
     def test_empty_base_name(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             with self.assertRaises(ValueError):
                 _ = hdf_utils.create_indexed_group(h5_f, '    ')
         os.remove(file_path)
@@ -783,7 +795,7 @@ class TestCreateIndexedGroup(TestSimple):
 
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             with self.assertRaises(TypeError):
                 _ = hdf_utils.create_indexed_group(h5_f, 1.2343)
         os.remove(file_path)
@@ -800,7 +812,7 @@ class TestCreateResultsGroup(TestSimple):
     def helper_first(self, add_dash_to_name=False):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=[1, 2, 3])
             if add_dash_to_name:
                 h5_group = hdf_utils.create_results_group(h5_dset, 'Some-Tool')
@@ -824,7 +836,7 @@ class TestCreateResultsGroup(TestSimple):
     def test_second(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=[1, 2, 3])
             h5_group = hdf_utils.create_results_group(h5_dset, 'Tool')
             self.assertIsInstance(h5_group, h5py.Group)
@@ -842,7 +854,7 @@ class TestCreateResultsGroup(TestSimple):
     def test_empty_tool_name(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset('Main', data=[1, 2, 3])
             with self.assertRaises(ValueError):
                 _ = hdf_utils.create_results_group(h5_dset, '   ')
@@ -854,7 +866,7 @@ class TestCreateResultsGroup(TestSimple):
 
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             with self.assertRaises(TypeError):
                 _ = hdf_utils.create_results_group(h5_f, 'Tool')
 
@@ -890,7 +902,7 @@ class TestLinkAsMain(TestSimple):
     def test_pos_args_not_h5_dset(self):
         file_path = 'link_as_main.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset("Blah", data=[1, 2, 3, 4])
             with self.assertRaises(TypeError):
                 hdf_utils.link_as_main("h5_main", 1.234, -2, False, {"h5_spec_vals": 2.432})
@@ -912,7 +924,7 @@ class TestLinkAsMain(TestSimple):
     def test_pos_args_not_h5_dset(self):
         file_path = 'link_as_main.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset = h5_f.create_dataset("Blah", data=[1, 2, 3, 4])
             with self.assertRaises(TypeError):
                 hdf_utils.link_as_main("h5_main", 1.234, -2, False, {"h5_spec_vals": 2.432})
@@ -934,7 +946,7 @@ class TestLinkAsMain(TestSimple):
     def test_ind_vals_not_same_shape(self):
         file_path = 'link_as_main.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_main = h5_f.create_dataset("Blah", data=np.zeros((3, 5), dtype=np.uint16))
             h5_pos_inds = h5_f.create_dataset("P_I", data=np.zeros((7, 2), dtype=np.uint16))
             h5_pos_vals = h5_f.create_dataset("P_V", data=np.zeros((3, 2), dtype=np.uint16))
@@ -945,7 +957,7 @@ class TestLinkAsMain(TestSimple):
 
         data_utils.delete_existing_file(file_path)
 
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_main = h5_f.create_dataset("Blah", data=np.zeros((3, 5), dtype=np.uint16))
             h5_pos_inds = h5_f.create_dataset("P_I", data=np.zeros((3, 2), dtype=np.uint16))
             h5_pos_vals = h5_f.create_dataset("P_V", data=np.zeros((3, 2), dtype=np.uint16))
@@ -959,7 +971,7 @@ class TestLinkAsMain(TestSimple):
     def helper_test(self, quant_units_specified):
         file_path = 'link_as_main.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_raw_grp = h5_f.create_group('Raw_Measurement')
 
             num_rows = 3
@@ -1029,7 +1041,7 @@ class TestCopyAttributes(TestSimple):
 
     def test_not_h5_dset(self):
         temp_path = 'copy_attributes.h5'
-        with h5py.File(temp_path) as h5_f:
+        with h5py.File(temp_path, mode='w') as h5_f:
             h5_grp = h5_f.create_group('Blah')
             with self.assertRaises(TypeError):
                 hdf_utils.copy_attributes(h5_grp, np.arange(4))
@@ -1044,7 +1056,7 @@ class TestCopyAttributes(TestSimple):
         also_easy_attr = {'N_numbers': [1, -53.6, 0.000463]}
         hard_attrs = {'N_strings': np.array(['a', 'bc', 'def'], dtype='S')}
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_f.attrs.update(easy_attrs)
             h5_f.attrs.update(also_easy_attr)
             h5_f.attrs.update(hard_attrs)
@@ -1064,7 +1076,7 @@ class TestCopyAttributes(TestSimple):
         also_easy_attr = {'N_numbers': [1, -53.6, 0.000463]}
         hard_attrs = {'N_strings': np.array(['a', 'bc', 'def'], dtype='S')}
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_group = h5_f.create_group('Group')
             h5_group.attrs.update(easy_attrs)
             h5_group.attrs.update(also_easy_attr)
@@ -1084,7 +1096,7 @@ class TestCopyAttributes(TestSimple):
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(5, 7)
         easy_attrs = {'1_string': 'Current', '1_number': 35.23}
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_source.attrs.update(easy_attrs)
             h5_dset_sink = h5_f.create_dataset('Sink', data=data)
@@ -1106,7 +1118,7 @@ class TestCopyAttributes(TestSimple):
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(5, 7)
         easy_attrs = {'1_string': 'Current', '1_number': 35.23}
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_source.attrs.update(easy_attrs)
             reg_refs = {'even_rows': (slice(None), slice(0, None, 2)),
@@ -1131,7 +1143,7 @@ class TestCopyAttributes(TestSimple):
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(5, 7)
         easy_attrs = {'1_string': 'Current', '1_number': 35.23}
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_source.attrs.update(easy_attrs)
             h5_dset_sink = h5_f.create_dataset('Sink', data=data)
@@ -1161,7 +1173,7 @@ class TestCopyAttributes(TestSimple):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(5, 7)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_dest = h5_f.create_dataset('Sink', data=data[:-1, :-1])
             reg_refs = {'even_rows': (slice(0, None, 2), slice(None)),
@@ -1182,7 +1194,7 @@ class TestCopyMainAttributes(TestSimple):
         file_path = 'test.h5'
         main_attrs = {'quantity': 'Current', 'units': 'nA'}
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Main_01', data=[1, 23])
             h5_dset_source.attrs.update(main_attrs)
             h5_group = h5_f.create_group('Group')
@@ -1195,7 +1207,7 @@ class TestCopyMainAttributes(TestSimple):
     def test_no_main_attrs(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Main_01', data=[1, 23])
             h5_group = h5_f.create_group('Group')
             h5_dset_sink = h5_group.create_dataset('Main_02', data=[4, 5])
@@ -1206,7 +1218,7 @@ class TestCopyMainAttributes(TestSimple):
     def test_wrong_objects(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Main_01', data=[1, 23])
             h5_group = h5_f.create_group('Group')
             with self.assertRaises(TypeError):
@@ -1224,7 +1236,7 @@ class TestCreateEmptyDataset(TestSimple):
 
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
             with self.assertRaises(TypeError):
                 _ = hdf_utils.create_empty_dataset(h5_dset_source, np.arange(15), 'Duplicate')
@@ -1247,7 +1259,7 @@ class TestCreateEmptyDataset(TestSimple):
         existing_attrs = {'a': 1, 'b': 'Hello'}
         easy_attrs = {'1_string': 'Current', '1_number': 35.23}
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
             h5_dset_source.attrs.update(existing_attrs)
             h5_duplicate = hdf_utils.create_empty_dataset(h5_dset_source, np.float16, 'Duplicate', new_attrs=easy_attrs)
@@ -1267,7 +1279,7 @@ class TestCreateEmptyDataset(TestSimple):
         existing_attrs = {'a': 1, 'b': 'Hello'}
         easy_attrs = {'1_string': 'Current', '1_number': 35.23}
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
             h5_dset_source.attrs.update(existing_attrs)
             h5_group = h5_f.create_group('Group')
@@ -1289,7 +1301,7 @@ class TestCreateEmptyDataset(TestSimple):
         data_utils.delete_existing_file(file_path)
         data = np.random.rand(5, 7)
         main_attrs = {'quantity': 'Current', 'units': 'nA'}
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=data)
             h5_dset_source.attrs.update(main_attrs)
             reg_refs = {'even_rows': (slice(0, None, 2), slice(None)),
@@ -1310,7 +1322,7 @@ class TestCreateEmptyDataset(TestSimple):
     def test_existing_dset_name(self):
         file_path = 'test.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
             _ = h5_f.create_dataset('Existing', data=[4, 5, 6])
             if sys.version_info.major == 3:
@@ -1331,33 +1343,191 @@ class TestCheckAndLinkAncillary(TestSimple):
         with self.assertRaises(TypeError):
             hdf_utils.check_and_link_ancillary(np.arange(5), ['Spec'])
 
-    def test_not_string_tuple(self):
-        file_path = 'check_and_link_ancillary.h5'
-        data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
-            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
-            with self.assertRaises(TypeError):
-                hdf_utils.check_and_link_ancillary(h5_dset_source, 'Spec')
-        os.remove(file_path)
-
     def test_h5_main_not_dset(self):
         file_path = 'check_and_link_ancillary.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
             with self.assertRaises(TypeError):
-                hdf_utils.check_and_link_ancillary(h5_dset_source, ['Spec'], h5_main="not_a_dataset")
+                hdf_utils.check_and_link_ancillary(h5_dset_source, ['Spec'],
+                                                   h5_main="not_a_dataset")
         os.remove(file_path)
 
-    def test_anc_refs_not_list_of_hdf(self):
+    def test_one_dset_to_name(self):
         file_path = 'check_and_link_ancillary.h5'
         data_utils.delete_existing_file(file_path)
-        with h5py.File(file_path) as h5_f:
+        with h5py.File(file_path, mode='w') as h5_f:
             h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
-            with self.assertRaises(TypeError):
-                hdf_utils.check_and_link_ancillary(h5_dset_source, ['Spec'], anc_refs=h5_dset_source)
+            h5_dset_2 = h5_f.create_dataset('Other', data=[1, 2, 3])
+            att_name = 'root'
+            expected = h5_dset_2
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_name,
+                                               h5_main=None, anc_refs=expected)
+            # Only one attribute expected
+            self.assertEqual(len(h5_dset_source.attrs.keys()), 1)
+            self.assertTrue(att_name in h5_dset_source.attrs.keys())
+            actual = h5_dset_source.attrs[att_name]
+            self.assertIsInstance(actual, h5py.Reference)
+            self.assertEqual(h5_f[actual], expected)
         os.remove(file_path)
 
+    def test_many_objs_to_many_names(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        with h5py.File(file_path, mode='w') as h5_f:
+            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
+            h5_dset_2 = h5_f.create_dataset('Other', data=[1, 2, 3])
+            h5_grp = h5_f.create_group('Blah')
+            att_names = ['Meh', 'Wah']
+            expected = [h5_dset_2, h5_grp]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=None, anc_refs=expected)
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+    def test_objs_and_refs_to_names(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        with h5py.File(file_path, mode='w') as h5_f:
+            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
+            h5_dset_2 = h5_f.create_dataset('Other', data=[1, 2, 3])
+            h5_grp = h5_f.create_group('Blah')
+            att_names = ['Meh', 'Wah']
+            expected = [h5_dset_2, h5_grp.ref]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=None, anc_refs=expected)
+            expected = [h5_dset_2, h5_grp]
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+    def test_too_many_names(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        with h5py.File(file_path, mode='w') as h5_f:
+            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
+            h5_dset_2 = h5_f.create_dataset('Other', data=[1, 2, 3])
+            _ = h5_f.create_group('Blah')
+            att_names = ['Meh', 'Wah']
+            expected = [h5_dset_2]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=None, anc_refs=expected)
+            att_names = [att_names[0]]
+            print(list(h5_dset_source.attrs.keys()), att_names)
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+    def test_too_many_ancs(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        with h5py.File(file_path, mode='w') as h5_f:
+            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
+            h5_dset_2 = h5_f.create_dataset('Other', data=[1, 2, 3])
+            h5_grp = h5_f.create_group('Blah')
+            att_names = ['Meh']
+            expected = [h5_dset_2, h5_grp]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=None, anc_refs=expected)
+            expected = [expected[0]]
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+    def test_no_anc_refs_provided(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        with h5py.File(file_path, mode='w') as h5_f:
+            h5_dset_source = h5_f.create_dataset('Source', data=[1, 2, 3])
+            with self.assertRaises(ValueError):
+                hdf_utils.check_and_link_ancillary(h5_dset_source, ['M', 'Wa'],
+                                                   h5_main=None, anc_refs=None)
+        os.remove(file_path)
+
+    def test_linking_main_legit_anc_names(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        shutil.copy(data_utils.std_beps_path, file_path)
+        with h5py.File(file_path, mode='r+') as h5_f:
+            h5_grp = h5_f['Raw_Measurement']
+            h5_dset_source = h5_grp['Ancillary']
+            h5_main = h5_grp['source_main']
+            att_names = ['Spectroscopic_Values', 'Position_Indices']
+            expected = [h5_grp[name] for name in att_names]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=h5_main, anc_refs=None)
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+    def test_h5_main_non_dset_anc_names(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        shutil.copy(data_utils.std_beps_path, file_path)
+        with h5py.File(file_path, mode='r+') as h5_f:
+            h5_grp = h5_f['Raw_Measurement']
+            h5_dset_source = h5_grp['Ancillary']
+            h5_main = h5_grp['source_main']
+            att_names = ['Spectroscopic_Values', 'quantity']
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=h5_main, anc_refs=None)
+            # Should NOT copy the quantity attribute since it is not a dset
+            att_names = [att_names[0]]
+            expected = [h5_grp['Spectroscopic_Values']]
+            self.assertEqual(set(att_names),
+                             set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+
+"""
+    def test_linking_main_plus_other_dsets(self):
+        file_path = 'check_and_link_ancillary.h5'
+        data_utils.delete_existing_file(file_path)
+        shutil.copy(data_utils.std_beps_path, file_path)
+        with h5py.File(file_path, mode='r+') as h5_f:
+            h5_grp = h5_f['Raw_Measurement']
+            h5_dset_source = h5_grp['Ancillary']
+            h5_main = h5_grp['source_main']
+            att_names = ['Spectroscopic_Values', 'Position_Indices', 'X', 'Y']
+            expected = [h5_grp[name] for name in att_names]
+            hdf_utils.check_and_link_ancillary(h5_dset_source, att_names,
+                                               h5_main=h5_main,
+                                               anc_refs=expected[2:])
+            for key, val in h5_dset_source.attrs.items():
+                print(key, val)
+            self.assertEqual(len(h5_dset_source.attrs.keys()), len(att_names))
+            self.assertEqual(set(att_names), set(h5_dset_source.attrs.keys()))
+            for name, exp_val in zip(att_names, expected):
+                actual = h5_dset_source.attrs[name]
+                self.assertIsInstance(actual, h5py.Reference)
+                self.assertEqual(h5_f[actual], exp_val)
+        os.remove(file_path)
+"""
+
 
 if __name__ == '__main__':
     unittest.main()
