@@ -11,7 +11,7 @@ import sys
 import h5py
 import numpy as np
 import dask.array as da
-from tests.processing.data_utils import validate_aux_dset_pair, delete_existing_file
+from .data_utils import validate_aux_dset_pair, delete_existing_file
 sys.path.append("../../pyUSID/")
 from pyUSID.io import ArrayTranslator, write_utils, hdf_utils, USIDataset
 
@@ -137,7 +137,7 @@ class TestArrayTranslator(unittest.TestCase):
 class TestBaseOperations(TestArrayTranslator):
 
     def test_preexisting_file_OK(self):
-        with h5py.File(file_path) as _:
+        with h5py.File(file_path, mode='w') as _:
             pass
         self.base_translation_tester(main_dset_as_dask=False, extra_dsets_type='numpy', use_parm_dict=False)
 
