@@ -10,7 +10,7 @@ import sys
 from PIL import Image
 import h5py
 import numpy as np
-from tests.processing.data_utils import validate_aux_dset_pair, delete_existing_file
+from .data_utils import validate_aux_dset_pair, delete_existing_file
 sys.path.append("../../pyUSID/")
 from pyUSID.io import ImageTranslator, hdf_utils, USIDataset
 from pyUSID.io.image import read_image
@@ -266,7 +266,7 @@ class TestFile(TestImageTranslator):
             _ = translator.translate('no_such_file.png')
 
     def test_output_h5_file_already_exists(self):
-        with h5py.File(image_path.replace('.png', '.h5')) as _:
+        with h5py.File(image_path.replace('.png', '.h5'), mode='w') as _:
             pass
         with self.assertRaises(FileExistsError):
             translator = ImageTranslator()
